{% extends "comments/base.html" %}
{% load i18n %}

{% block title %}{% trans "Approve a comment" %}{% endblock %}

{% block content %}
  <h1>{% trans "Really make this comment public?" %}</h1>
  <blockquote>{{ comment|linebreaks }}</blockquote>
  <form action="." method="post">
    {% if next %}<input type="hidden" name="next" value="{{ next }}" id="next" />{% endif %}
    <p class="submit">
      <input type="submit" name="submit" value="{% trans "Approve" %}" /> or <a href="{{ comment.get_absolute_url }}">cancel</a>
    </p>
  </form>
{% endblock %}
