{% extends "admin/base_site.html" %}

{% block content %}

<div id="content-main">

<form action="" method="post">

{% if form.errors %}
<p class="errornote">Your template had {{ form.errors|length }} error{{ form.errors|pluralize }}:</p>
{% endif %}

<fieldset class="module aligned">
<div class="form-row{% if form.errors.site %} error{% endif %} required">
    {{ form.errors.site }}
    <h4><label for="id_site">{{ form.site.label }}:</label> {{ form.site }}</h4>
</div>
<div class="form-row{% if form.errors.template %} error{% endif %} required">
    {{ form.errors.template }}
    <h4><label for="id_template">{{ form.template.label }}:</label> {{ form.template }}</h4>
</div>
</fieldset>

<div class="submit-row">
    <input type="submit" value="Check for errors" class="default" />
</div>

</form>
</div>

{% endblock %}
